define([
], function(
) {
    "use strict";

    function applyClasses (element, classes, remove) {
        if(!classes)
            return;
        if(typeof classes === 'string')
            classes = classes.split(' ').filter(function(item){return !!item;});
        if( element.classList )
            element.classList[remove ? 'remove' : 'add'].apply(element.classList, classes);
        else {
            // IE11 and SVG elements apparently :-/
            var classesToRemove
              , seen
              , filterFunc
              ;
            if(remove) {
                classesToRemove = new Set(classes);
                filterFunc = function(item) {
                    return !classesToRemove.has(item);
                };
            }
            else {
                seen = new Set();
                element.setAttribute('class', element.getAttribute('class')
                        + (' ' + classes.join(' '))
                );

                filterFunc = function(item) {
                    if(seen.has(item))
                        return false;
                    seen.add(item);
                    return true;

                };
            }
            element.setAttribute('class', element.getAttribute('class')
                                                 .split(' ')
                                                 .filter(filterFunc)
                                                 .join(' ')
                                );

        }
    }

    function insertElement(into, element, pos) {
        var children = into.children || into.childNodes
          , append = children.length
          ;
        if(pos === undefined || pos > append)
            pos = append;
        else if(pos < 0) {
            pos = children.length + pos;
            if(pos < 0)
                pos = 0;
        }
        if(pos === append)
            into.appendChild(element);
        else
            into.insertBefore(element, children[pos]);
    }

    return {
        applyClasses: applyClasses
      , insertElement: insertElement
    };
});
